clear
cd('/Users/zachbrown/Documents/gmm_simulation')

% Include path to Stan installation
addpath('/Users/zachbrown/Applications/MATLAB/stan/cmdstan-2.9.0')
addpath('/Users/zachbrown/Applications/MATLAB/stan/MatlabProcessManager-master')
addpath('/Users/zachbrown/Applications/MATLAB/stan/MatlabStan-2.7.0.0')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulate Data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = 10000;

% Parameters
beta = [1;2;3];

% Correlated error
sigma = [15,0.9;0.9,15];
eps = mvnrnd([0,0],sigma,N);

% Exogenous vars
Z = [ones(N,1), random('normal', 5, 3,[N,1]), random('normal', 5, 3,[N,1])];

% Endogenous vars
x_endo = Z(:,2) + eps(:,1);

X = [Z(:,1), x_endo, Z(:,3)];

% Outcome
Y = X*beta + eps(:,2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run OLS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%est_ols = regress(Y,X);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run IV
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
est_iv = inv(X'*Z*inv(Z'*Z)*Z'*X)*X'*Z*inv(Z'*Z)*Z'*Y;

[est_ols, est_iv]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run GMM in Matlab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c = [1;1;1];
number=100;
K=size(Z,2);
[est_gmm,stde_gmm,W_out]=gmm_est(c,Y,X,Z,number,K);

disp([beta est_ols est_iv c est_gmm stde_gmm])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run MCMC GMM via Stan
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c = [1;1;1];

% Use optimal weight matrix from above
W = W_out;

data = struct('N',N,'K',size(X,2),'X', X,'Z',Z,'W',W,'Y',Y);
initial = struct('beta',c);
    
delete('temp.data.R','temp.init.R','stan_gmm','stan_gmm.hpp');
fit_mcmc = stan('file','stan_gmm.stan','data',data,'iter',4000, ...
    'init',initial,'file_overwrite',true,'verbose',true, ...
    'chains',4,'refresh',10);

% Get estimates when finished
fit_mcmc.block();
%print(est_mcmc)
est_mcmc_samples = fit_mcmc.extract('permuted',true).beta;
%hist(est_mcmc_samples(:,1))
est_mcmc=mean(est_mcmc_samples)';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run GMM using optimization via Stan (with autodiff)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c = [1;1;1];

delete('temp.data.R','temp.init.R','stan_gmm','stan_gmm.hpp');
fit_stangmm = stan('file','stan_gmm.stan','data',data,'iter',200,'init',initial,'file_overwrite',true,'verbose',true,'refresh',10,'method','optimize');

% Get estimates when finished
fit_stangmm.block();
%print(fit_stangmm)
est_stangmm = fit_stangmm.sim.samples.beta;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compare Results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('    True        IV       GMM       MCMC     stan_GMM')
disp([beta est_iv est_gmm est_mcmc est_stangmm])



